package install.doc.examples;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.OutputStream;
import java.net.URL;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.MTOMFeature;

import com.elpical.jclaro.ws.client.Account;
import com.elpical.jclaro.ws.client.JobImpl;
import com.elpical.jclaro.ws.client.JobImplService;

/**
 * Example of how to use the Claro JobClient webservice 
 * 
 *
 */
public class ElpicalClaroWebservices_example {
	private static JobImpl port = null;
	
	static {
		// NOTE: This code prevents this error:
		// SSLHandshakeException: No subject alternative names present
		// 
		// Or this error: No subject alternative names matching IP address
		HttpsURLConnection.setDefaultHostnameVerifier(new HostnameVerifier() {
			/*
			 * (non-Javadoc)
			 * 
			 * @see javax.net.ssl.HostnameVerifier#verify(java.lang.String,
			 * javax.net.ssl.SSLSession)
			 */
			public boolean verify(String hostname, SSLSession session) {
				boolean result = false;
	
				// NOTE: Simply return true to allow everyone to use this example, but in production 
				// code this should NOT be the way to do it.
				result = true;
	
				return result;
			}
		});
	}

	/**
	 * Get the webservice port 
	 * 
	 * @param hostname
	 * @param portnumber
	 * @param timeout
	 * @param asMTOM
	 * @return
	 * @throws Exception
	 */
	public static JobImpl getPort(String hostname, int portnumber,int timeout,boolean asMTOM) throws Exception {
		URL url = null;
		URL baseUrl;
		baseUrl = com.elpical.jclaro.ws.client.JobImplService.class.getResource(".");
		url = new URL(baseUrl, "https://"+hostname+":"+portnumber+"/jclarojaxws/job?wsdl");
		JobImplService anis = new JobImplService(url, new QName("http://server.ws.jclaro.elpical.com/", "JobImplService"));
		
		if (asMTOM)
			port = anis.getJobImplPort(new MTOMFeature(true,8443));
		else
			port = anis.getJobImplPort();

		if (port==null)
			throw new Exception("JobClient getPort can not be retrieved");
		
		int i_timeout = 5; // in sec.		
		timeout = i_timeout * 1000;
		Map<String, Object> ctxt1 = ((BindingProvider)port).getRequestContext();
		ctxt1.put("com.sun.xml.ws.connect.timeout", timeout);

		// setReadTimeout()
		Map<String, Object> ctxt2 = ((BindingProvider)port).getRequestContext();
		ctxt2.put("com.sun.xml.ws.request.timeout", timeout);
		return port;
	}
	
	/**
	 * Get a ClientID
	 * 
	 * @param port
	 * @return
	 */
	private static String getClientID(JobImpl port, byte[] userToken) {
		String ClientID = null;
		
		try {
			File ClientIDfile = new File("clid.txt");
			BufferedReader fr = null;
			BufferedWriter fw = null;
			
			if (ClientIDfile.exists()) {
				try {
					fr = new BufferedReader(new FileReader(ClientIDfile));
					String[] line_arr = fr.readLine().split("clid=");
					ClientID = line_arr[1];
				}
				catch (Exception ew) {
		        	System.out.println(ew.toString() + " ElpicalClaroWebservices_example getClientID 1");
		        }
				finally {
					try {
						fr.close();
					}
					catch (Exception e) {
			        	System.out.println(e.toString() + " ElpicalClaroWebservices_example getClientID 2");
			        }
				}
			}
			
			if (ClientID==null) {
				ClientID = port.getClientID(userToken);
				
				try {
					fw =  new BufferedWriter(new FileWriter(ClientIDfile));
					fw.write("clid="+ClientID+"\n");
				}
				catch (Exception ew) {
		        	System.out.println(ew.toString() + " ElpicalClaroWebservices_example getClientID 3");
		        }
				finally {
					try {
						fw.close();
					}
					catch (Exception e) {
			        	System.out.println(e.toString() + " ElpicalClaroWebservices_example getClientID 4");
			        }
				}	        
			} else {
				port.checkClientID(userToken, ClientID);
				
				checkError(userToken);
			}
		} catch (Exception xLast) {
			System.out.println(xLast.getMessage());
		}
		
		return ClientID;
	}
	
	/**
	 * Check and report any error currently set in the JC WS.
	 */
	private static void checkError(byte[] userToken) throws Exception {
		int errorCode = -1;
		String errorMsg = null;
		
		errorCode = port.getErrorCodeAsInteger(userToken);
		errorMsg = port.getErrorMessage(userToken);
		
		if (errorCode != -1) {
			// Handle error here, for example:
			
			//System.out.println(errorMsg);
			//throw new Exception(errorMsg);
		}
	}
	
	/**
	 * @param args
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {
		System.out.println("Starting webservices example");
		
		byte[] userToken = null;

		//WebServices properties 
//		System.setProperty("javax.xml.stream.XMLEventFactory","com.sun.xml.stream.events.ZephyrEventFactory");
//		System.setProperty("javax.xml.stream.XMLInputFactory","com.sun.xml.stream.ZephyrParserFactory");
//		System.setProperty("javax.xml.stream.XMLOutputFactory","com.sun.xml.stream.ZephyrWriterFactory");

		try {
			////
			
			// This is the only way to work with a self-signed certificate.
			
			// **** WARNING ****
			// Trusting all certificates allows man-in-the-middle attacks 
			// and is completely insecure.
			
			// Create a trust manager that does not validate certificate chains
			TrustManager[] trustAllCerts = new TrustManager[] { 
				new X509TrustManager() {
					/*
					 * (non-Javadoc)
					 * 
					 * @see javax.net.ssl.X509TrustManager#getAcceptedIssuers()
					 */
					public java.security.cert.X509Certificate[] getAcceptedIssuers() {
						return null;
					}
			
					public void checkClientTrusted(java.security.cert.X509Certificate[] certs, String authType) {
					}
			
					public void checkServerTrusted(java.security.cert.X509Certificate[] certs, String authType) {
					}
			} };
			
			// Install the all-trusting trust manager
			try {
			    SSLContext sc = SSLContext.getInstance("SSL");
			    
			    // NOTE: Set the second parameter to null to disable trust-all.
			    // NOTE: This applies when a certificate signed by a CA is used.
			    sc.init(null, trustAllCerts, new java.security.SecureRandom());
			    HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());
			} catch (Exception xLast) {
				Logger.getGlobal().severe("Exception: " + xLast.getMessage());
			}

			////
			
			JobImpl port = getPort("192.168.178.199",8443,10,true);			
			
			// Optionally you can enter a username and password here as 
			// configured in ClaroAdministration's user accounts.
			Account acc = port.login("", "");
			
			if (acc == null) {
				throw new Exception("Login failed: Account is null!");
			} else {
				userToken = acc.getEncryptedToken();
				
				if (userToken == null) {
					throw new Exception("Login failed: userToken is null!");
				}
			}
			
			String ClientID = getClientID(port, userToken);
			File imgfile = new File("/temp/c0001.jpg"); 
			File jobimgfile = new File("/temp/c0001.job");
			List<String> channels = null;
			
			channels = port.getChannels(userToken);
			checkError(userToken);
			
			String channel = channels.get(0);
			int jobid = 1;
			DataHandler dhimg = new DataHandler(new FileDataSource(imgfile));
			DataHandler dhjob = new DataHandler(new FileDataSource(jobimgfile));
			System.out.print("processing file");
			
			port.processImage(userToken, dhjob,imgfile.getName(),dhimg,"c0001_return",channel,ClientID,jobid);			
			checkError(userToken);
			
			for(;;) {
				Thread.sleep(1000);
				
				int jobid_ready = port.getAnyJobReady(userToken, ClientID);
				checkError(userToken);
				
				if (jobid_ready == jobid) {
					System.out.println("ready");
					break;
				}
				System.out.print(".");
			}
			
			System.out.print("retrieving jobfile from server...");
			
			DataHandler dhjobret = (DataHandler)port.getJobFile(userToken, ClientID,jobid);
			checkError(userToken);
			
			File jobfile_ret = new File("/temp/c0001_return.job");
			FileOutputStream fos = new FileOutputStream(jobfile_ret);
			OutputStream output = new BufferedOutputStream(fos);
			
			try {
				dhjobret.writeTo(output);
			}
			catch (Exception e) {
	        	System.out.println(e.toString() + " ElpicalClaroWebservices_example main 1");
	        }
			finally {
				try {
					output.close();
				}
				catch (Exception e) {
		        	System.out.println(e.toString() + " ElpicalClaroWebservices_example main 2");
		        }
				
				try {
					fos.close();
				}
				catch (Exception e) {
		        	System.out.println(e.toString() + " ElpicalClaroWebservices_example main 3");
		        }
			}

			System.out.print("done\nretrieving imagefile from server...");
			
			DataHandler dhimgret = (DataHandler)port.getImageFile(userToken, ClientID,jobid,"c0001_return.jpeg", false);
			checkError(userToken);
			
			File imgfile_ret = new File("/temp/c0001_return.jpeg");
			fos = new FileOutputStream(imgfile_ret);
			output = new BufferedOutputStream(fos);
			
			try {
				dhimgret.writeTo(output);
			}
			catch (Exception e) {
	        	System.out.println(e.toString() + " ElpicalClaroWebservices_example main 4");
	        }
			finally {
				try {
					output.close();
				}
				catch (Exception e) {
		        	System.out.println(e.toString() + " ElpicalClaroWebservices_example main 5");
		        }
				
				try {
					fos.close();
				}
				catch (Exception e) {
		        	System.out.println(e.toString() + " ElpicalClaroWebservices_example main 6");
		        }
			}
			
			System.out.print("done\ncleaning up server...");
			
			port.removeJob(userToken, ClientID,jobid);
			checkError(userToken);
			
			System.out.println("done");
		} catch (Exception e) {
			System.out.println(e.getMessage());
		}
		
		System.out.println("Finshised webservices example");
	}
}