package install.doc.examples;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.URL;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.util.List;

import javax.crypto.Cipher;
import javax.xml.namespace.QName;

import com.elpical.jclaro.admin.ws.client.Account;
import com.elpical.jclaro.admin.ws.client.AdminChannel;
import com.elpical.jclaro.admin.ws.client.AdminConnector;
import com.elpical.jclaro.admin.ws.client.AdminConnectorService;

public class ExampleAdminClient {

	public ExampleAdminClient(String[] args) throws Exception {

		String ip;
		String port;
		String user;
		String password;
		if (args != null && args.length == 4) {
			ip = args[0];
			port = args[1];
			user = args[2];
			password = args[3];
		} else {
			BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
			System.out.print("connect to host: ");
			ip = reader.readLine();
			System.out.print("port nr: ");
			port = reader.readLine();
			System.out.print("username: ");
			user = reader.readLine();
			System.out.print("password: ");
			password = reader.readLine();
		}

		System.out.println("\nConnecting to    : http://" + ip + ":" + port + "/AdminConnector/AdminConnector?WSDL");
		System.out.println("username/password: " + user + "/" + password);

		KeyPairGenerator keyGen = KeyPairGenerator.getInstance("RSA");
		keyGen.initialize(1024);
		KeyPair pair = keyGen.generateKeyPair();
		PrivateKey privateKey = pair.getPrivate();
		PublicKey publicKey = pair.getPublic();
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		ObjectOutputStream stream = new ObjectOutputStream(out);
		stream.writeObject(publicKey);

		byte[] clientPublicKeyBytes = out.toByteArray();

		URL url = new URL(new URL("http://" + ip + ":" + port), "AdminConnector/AdminConnector?WSDL");

		AdminConnectorService service = new AdminConnectorService(url, new QName("http://server.ws.admin.jclaro.elpical.com/", "AdminConnectorService"));
		System.out.println("get connector");
		AdminConnector connector = service.getAdminConnectorPort();

		byte[] serverPublicKeyBytes = connector.getPublicKey();
		ObjectInputStream in = new ObjectInputStream(new ByteArrayInputStream(serverPublicKeyBytes));
		PublicKey serverPublicKey = (PublicKey) in.readObject();

		final Cipher encryptCipher = Cipher.getInstance("RSA");
		encryptCipher.init(Cipher.ENCRYPT_MODE, serverPublicKey);

		byte[] encryptedPassword = new byte[0];
		if (password != null && password.length() > 0) {
			encryptedPassword = encryptCipher.doFinal(password.getBytes());
		}

		Account account = connector.login(user, encryptedPassword, clientPublicKeyBytes);
		if (account == null) {
			throw new Exception("Login Error");
		}

		final Cipher decryptCipher = Cipher.getInstance("RSA");
		decryptCipher.init(Cipher.DECRYPT_MODE, privateKey);
		byte[] loginToken = decryptCipher.doFinal(account.getEncryptedToken());

		System.out.println("Succesfull login. Encrypted login token: " + new String(account.getEncryptedToken() + " Decrypted login token: " + new String(loginToken)));

		byte[] encryptedToken = encryptCipher.doFinal(loginToken);
		System.out.println("********* Ask for list of channels with encrypted token: " + new String(encryptedToken));
		System.out.println("*****************************************************************************************");
		List<AdminChannel> channels = connector.getChannels(encryptedToken);
		for (AdminChannel adminChannel : channels) {
			System.out.println("Channel: " + adminChannel.getName());
		}
	}

	public static void main(String[] args) {

		try {
			new ExampleAdminClient(args);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

}
